﻿using System;
using System.Drawing;

namespace WindowsFormsApplication1
{
    class BallObject
        : IDisposable
    {
        private static Color[] Colors = { Color.Green, Color.Red, Color.Blue, Color.Yellow, Color.Violet, Color.Indigo };
        private static int ColorIndex = 0;

        private static Random Rnd = new Random();
        public Vector Location;
        public Vector Speed;
        public float Mass;
        private Brush brush;

        /// <summary>
        /// Größe des Objekts => Kugel
        /// </summary>
        public static int Radius = 10;
        public BallObject(float mass = 1F)
        {
            // oben links in der Ecke
            this.Location = new Vector(2 * BallObject.Radius, 2 * BallObject.Radius, 0);
            this.Speed = new Vector();
            this.Mass = mass;
            this.brush = new SolidBrush(BallObject.Colors[BallObject.ColorIndex]);
            BallObject.ColorIndex = (BallObject.ColorIndex + 1) % BallObject.Colors.Length;
        }

        public void InitRandomSpeed()
        {
            this.Speed.X = (float)BallObject.Rnd.NextDouble();
            this.Speed.Y = (float)BallObject.Rnd.NextDouble();
            //this.Speed.Z = (float)BaseObject.Rnd.NextDouble();
            this.Speed.Z = 0;
        }

        /// <summary>
        /// Testet, ob die gegenwärtige Position
        /// innerhalb des x-y-Spielfeldes liegt.
        /// Ansonsten wird eine Reflexion
        /// an der starren Wand durchgeführt.
        /// </summary>
        /// <param name="border">Spielfeldrand</param>
        /// <returns>True: innerhalb</returns>
        public bool CheckBorder(RectangleF border)
        {
            RectangleF pos = new RectangleF(this.Location.X, this.Location.Y, 0, 0);
            pos.Inflate(BallObject.Radius, BallObject.Radius);
            bool crash = !border.Contains(pos);
            if (crash)
            {
                if (this.Location.X - BallObject.Radius < 0)
                {
                    float delta = this.Location.X - BallObject.Radius;
                    this.Location.X = BallObject.Radius + delta;
                    this.Speed.X = -this.Speed.X;
                }

                if (this.Location.X + BallObject.Radius > border.Width)
                {
                    float delta = border.Width - (this.Location.X + BallObject.Radius);
                    this.Location.X = border.Width - BallObject.Radius - delta;
                    this.Speed.X = -this.Speed.X;
                }

                if (this.Location.Y - BallObject.Radius < 0)
                {
                    float delta = this.Location.Y - BallObject.Radius;
                    this.Location.Y = BallObject.Radius + delta;
                    this.Speed.Y = -this.Speed.Y;
                }

                if (this.Location.Y + BallObject.Radius > border.Height)
                {
                    float delta = border.Height - (this.Location.Y + BallObject.Radius);
                    this.Location.Y = border.Height - BallObject.Radius - delta;
                    this.Speed.Y = -this.Speed.Y;
                }
            }

            return crash;
        }

        /// <summary>
        /// Zweidimensionaler elastischer Stoß
        /// </summary>
        /// <param name="o1"></param>
        /// <param name="o2"></param>
        /// <returns></returns>
        public static bool CheckCrash(BallObject o1, BallObject o2)
        {
            // https://de.wikipedia.org/wiki/Sto%C3%9F_(Physik)

            Vector dist = (o1.Location - o2.Location);
            float distance = dist.Betrag();
            if (distance >= 2 * BallObject.Radius)
            {
                return false;
            }

            Vector v1 = o1.Speed;
            Vector v2 = o2.Speed;
            float m1 = o1.Mass;
            float m2 = o2.Mass;

            Vector v11 = 2F * ((m1 * v1) + (m2 * v2)) / (m1 + m2) - v1;
            Vector v22 = 2F * ((m1 * v1) + (m2 * v2)) / (m1 + m2) - v2;

            o1.Speed = v11;
            o2.Speed = v22;

            return true;
        }

        internal void Draw(Graphics graphics)
        {
            RectangleF pos = new RectangleF(this.Location.X, this.Location.Y, 0, 0);
            pos.Inflate(BallObject.Radius, BallObject.Radius);
            graphics.FillEllipse(this.brush, pos);
        }

        internal void Step(float duration)
        {
            this.Location.ChangePosition(this.Speed, duration);
        }

        public void Dispose()
        {
            if (this.brush != null)
            {
                this.brush.Dispose();
            }
        }
    }
}
